import os
import sys
import subprocess
import grass.script.core as gcore

def get_executable():
    if 'GRASS_EXECUTABLE' in os.environ:
        executable = os.environ['GRASS_EXECUTABLE']
    else:
        executable = 'grass'
    if sys.platform.startswith('win'):
        executable = r'C:\OSGeo4W\grass70.bat'
        # executable = r'C:\Program Files (x86)\GRASS GIS 7.0.0\grass70.bat'
        # this can be avoided if GRASS executable is added to PATH
    elif sys.platform == 'darwin':
        # this have to be checked, maybe unix way is good enough
        executable = '/Applications/GRASS/GRASS-7.0.app/'
    return executable


def get_gisbase(grass_executable=None):
    if grass_executable is None:
        grass_executable = get_executable()
    else:
        grass_executable = os.path.expanduser(grass_executable)
    # query GRASS GIS itself for its GISBASE
    startcmd = [grass_executable, '--config', 'path']
    try:
        p = subprocess.Popen(startcmd, shell=False,
                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = p.communicate()
    except OSError as error:
        sys.exit("ERROR: Cannot find GRASS GIS start script"
                 " {cmd}: {error}".format(cmd=startcmd[0], error=error))
    if p.returncode != 0:
        sys.exit("ERROR: Issues running GRASS GIS start script"
                 " {cmd}: {error}"
                 .format(cmd=' '.join(startcmd), error=err))
    gisbase = out.strip(os.linesep)
    return gisbase


def init_runtime(gisbase=None, grass_executable=None):
    if gisbase is None:
        gisbase = get_gisbase(grass_executable)
    else:
        gisbase = os.path.expanduser(gisbase)
    mswin = sys.platform.startswith('win')
    # define PATH
    os.environ['PATH'] += os.pathsep + os.path.join(gisbase, 'bin')
    os.environ['PATH'] += os.pathsep + os.path.join(gisbase, 'scripts')
    if mswin:  # added for winGRASS
        os.environ['PATH'] += os.pathsep + os.path.join(gisbase, 'extrabin')

    # add addons to the PATH
    # copied and simplified from lib/init/grass.py
    if mswin:
        config_dirname = "GRASS7"
        config_dir = os.path.join(os.getenv('APPDATA'), config_dirname)
    else:
        config_dirname = ".grass7"
        config_dir = os.path.join(os.getenv('HOME'), config_dirname)
    addon_base = os.path.join(config_dir, 'addons')
    os.environ['GRASS_ADDON_BASE'] = addon_base
    if not mswin:
        os.environ['PATH'] += os.pathsep + os.path.join(addon_base, 'scripts')

    # define LD_LIBRARY_PATH
    if 'LD_LIBRARY_PATH' not in os.environ:
        os.environ['LD_LIBRARY_PATH'] = ''
    os.environ['LD_LIBRARY_PATH'] += os.pathsep + os.path.join(gisbase, 'lib')

    os.environ['GIS_LOCK'] = str(os.getpid())

    # Set GRASS_PYTHON and PYTHONPATH to find GRASS Python modules
    if not os.getenv('GRASS_PYTHON'):
        if sys.platform == 'win32':
            os.environ['GRASS_PYTHON'] = "python.exe"
        else:
            os.environ['GRASS_PYTHON'] = "python"
    
    path = os.getenv('PYTHONPATH')
    etcpy = os.path.join(gisbase, 'etc', 'python')
    if path:
        path = etcpy + os.pathsep + path
    else:
        path = etcpy
    os.environ['PYTHONPATH'] = path

    os.environ['GISBASE'] = gisbase


def init_data(dbase=None, location=None, mapset=None):
    if not 'GISBASE' in os.environ:
        init_runtime()
    # define GRASS Database
    # add your path to grassdata (GRASS GIS database) directory
    if dbase is None:
        if sys.platform.startswith('win'):
            # the following path is the default path on MS Windows
            gisdb = os.path.join(os.path.expanduser("~"), "Documents", "grassdata")
        else:
            gisdb = os.path.join(os.path.expanduser("~"), "grassdata")
    from grass.script.setup import write_gisrc
    gisrc = write_gisrc(dbase, location, mapset)
    os.environ['GISRC'] = gisrc
    return gisrc


import grass.script as gscript

def test():
    # example calls
    gscript.message('Current GRASS GIS 7 environment:')
    print gscript.gisenv()

    gscript.message('Available raster maps:')
    for rast in gscript.list_strings(type='raster'):
        print rast

    gscript.message('Available vector maps:')
    for vect in gscript.list_strings(type='vector'):
        print vect

    from grass.pygrass.vector.geometry import Point
    point1 = Point(635818.8, 221342.4)


rcfile = init_data(".../grassdata", "nc_spm_08_grass7", "user1")

test()

os.remove(rcfile)
